﻿using System;
using System.Xml;
using System.Linq;
using System.Data;
using System.Configuration;
using System.Collections.Generic;
using SqlUtils;
using Newtonsoft.Json;
using WebReports.Api.Common;
using WebReports.Api;

namespace ExagoExtensibility
{
    /// <summary>
    /// Custom Folder Management implementation that handles storing and retrieving report, folder, theme and document template definitions from a database.
    /// This example uses a User Defined Field (UDF) property to store entities by their database ID instead of their name and path
    /// See New-Database-Setup.sql.txt for the Database Schema used by this implementation 
    /// </summary>

    public class FolderManagement
    {
        /* Helper objects, classes and methods */
        private static DbObjectFactory.DataSourceType db = DbObjectFactory.GetDataSourceType(ConfigurationManager.ConnectionStrings["DefaultConnection"].ProviderName);
        private static string connectionString = ConfigurationManager.ConnectionStrings["DefaultConnection"].ConnectionString;
        private static char[] tokens = { '\\', '/' };   // folder and report name parsing delimiters

		/** 
		 * A recursive method to find an Entity's children
		 * @param ds is a DataSet of rows representing the folders and reports a user can access
         * @param parentId is the db ID of the Entity whose children are being found
         * @return a List of Entities
         */
		public static List<Entity> FindAllChildren(DataSet ds, string parentId)
        {
			List<Entity> entityList = new List<Entity>();	// the list that will be returned at any particular level of recursion

			DataRow dr; bool isLeaf; string rptGui; Entity e;   // inter-loop variable defintions
			for (int i = 0; i < ds.Tables[0].Rows.Count; i++)	// for each row in the dataset
			{
				dr = ds.Tables[0].Rows[i];  // for brevity, set row variable

				// if the row was marked as deleted because it was already processed, skip to next iteration of loop
				if (dr.RowState == DataRowState.Deleted)
					continue;

				if (String.Compare(dr.Field<string>("parentId"), parentId) == 0)	// if current row is that which we want to process for this method call
				{
					// create a new entity object from this data row
					isLeaf = Convert.ToBoolean(dr["leaf_flag"]);	// convenience variable because we'll use this more than once
					rptGui = dr.Field<string>("id");				// convenience variable because we'll use this more than once
					e = new Entity(rptGui, dr.Field<string>("name"), isLeaf, Convert.ToBoolean(dr["readOnly"]), dr.Field<string>("parentId"), dr.Field<int?>("sortOrder"), 
						dr.Field<string>("ownerId"), Convert.ToBoolean(dr["canRename"]), dr.Field<int?>("reportType"), dr.Field<int?>("partyType"));

					dr.Delete();	// mark this row as deleted, though it doesn't actually remove row from dataset (faster)

					if (!isLeaf)    // if this is a folder node (not a report), continue processing child level recursively 
						e.Children = FindAllChildren(ds, rptGui);

					entityList.Add(e);	// add new entity to list for this hierarchical level
				}
			}

			if (entityList.Count > 0)
				return entityList;

			return null;
		}

        /** 
		 * A recursive method to build XML representing the folders and reports a user can access
         * @param entities  List of Entities to create xml for
         * @param xmlDoc the XmlDocument to write to
         * @param root the xml node to write new nodes inside of
         * @return a string of xml
         */
        public static string WriteXml(List<Entity> entities, XmlDocument xmlDoc, XmlNode root)
        {
            foreach (Entity entity in entities)
            {
                XmlNode node = xmlDoc.CreateElement("entity");
                root.AppendChild(node);

				XmlAttribute attribute = xmlDoc.CreateAttribute("id");
                attribute.Value = entity.Id.ToString();
                node.Attributes.Append(attribute);

				XmlNode nameNode = xmlDoc.CreateElement("name");
				nameNode.InnerText = entity.Name;
				node.AppendChild(nameNode);

				XmlNode leafNode = xmlDoc.CreateElement("leaf_flag");
				leafNode.InnerText = entity.LeafFlag.ToString();
				node.AppendChild(leafNode);

				XmlNode roNode = xmlDoc.CreateElement("readonly_flag");
				roNode.InnerText = entity.ReadOnlyFlag.ToString();
				node.AppendChild(roNode);

				XmlNode udfNode = xmlDoc.CreateElement("udf");
				udfNode.InnerText = entity.UDF;
				node.AppendChild(udfNode);

				if (entity.Children != null)
                    WriteXml(entity.Children, xmlDoc, node);  // write nodes for each of this entity's children
			}
            return xmlDoc.InnerXml;
        }

        /** 
		 * Method called by Exago to populate the full list of folders and reports available to the active user
         * @param sessionInfo is the SessionInfo object as provided by Exago.  
         * @return XML representing the report and folder hierarchy
         */
        public static string GetReportList(SessionInfo sessionInfo)
        {
			DataSet ds = null;
			try
			{
                //Query the databse for all reports and folders the user has access to
                //The WHERE clause handles controlling which reports and folders the user has permission to access
                //Sub-Queries are used as a user may have access to a single report/folder more than once due to party permissions 
                //(ex. a report could be available to the party 'everyone' and to the users' specific 'company')
				DbSqlHandler sql = new DbSqlHandler(db, connectionString);
				sql.Stmt = string.Format(
					"SELECT id, name, leaf_flag, parentId, ownerId, reportType, " +
						"MIN( CASE WHEN readOnly=@ro THEN 1 ELSE 0 END ) AS readOnly, " +
						"MAX( sortOrder ) AS sortOrder, " +
						"MAX( CASE WHEN canRename=@canRename THEN 1 ELSE 0 END ) AS canRename, " +
                        "MAX(partyType) AS partyType " +
					"FROM ( " +
						"SELECT {0}.id, {0}.name, {0}.leaf_flag, {0}.ownerId, {0}.reportType, " +
                        "{1}.readOnly, {1}.parentId, {1}.sortOrder, {1}.canRename, {1}.partyType  " +
						"FROM {0} " +
						"INNER  JOIN {1} ON {0}.id={1}.reportId " +
						"INNER JOIN ( " +
							"SELECT reportId, MAX(partyType) AS partyType " +
							"FROM {1} " +
							"WHERE " +
								"{1}.isActive=@active AND " +
								permissionSqlClause +
							"GROUP BY reportId " +
						") AS b " +
						"ON {1}.reportId=b.reportId AND {1}.partyType=b.partyType " +
						"WHERE " +
							availabilitySqlClause +
							permissionSqlClause +
					") AS ReportListXML " +
					"GROUP BY id, name, leaf_flag, parentId, ownerId, reportType " +
					"ORDER BY sortOrder DESC, leaf_flag DESC, name ASC", DbTables.Reports, DbTables.ReportAccess);
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@ro", true, DbType.Boolean));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@canRename", true, DbType.Boolean));
                sql = setDefaultPermissionParams(sessionInfo, sql);
				sql = setDefaultAvailabilityParams(sql, EntityType.ReportFolder);
				ds = sql.ExecuteReadCmd();
			}
			catch
			{
				ds = null;
			}

			// If the user has no reports yet return this empty xml
			if (ds == null || ds.Tables[0].Rows.Count == 0)
                return "<entity></entity>";

			// build a linked list of the folders and reports as Entity objects
			List<Entity> sortedEntities = FindAllChildren(ds, null);

            //Create the XML document and root node then call WriteXML to traverse the linked list and create the final output
            XmlDocument xmlDoc = new XmlDocument();
            XmlNode rootNode = xmlDoc.CreateElement("ExagoReports");
            xmlDoc.AppendChild(rootNode);
			return WriteXml(sortedEntities, xmlDoc, rootNode);
		}

        /** 
		 * Method called by Exago to retrieve the Xml definition of a report
		 * @param sessionInfo is the SessionInfo object as provided by Exago.  
         * @param reportName is the fully qualified path of the queried report.  
         * @param udf is the json string set by GetReportListXml of the queried report.
         * @return XML representing the report definition or null if error
         */
        public static string GetReport(SessionInfo sessionInfo, string reportName, string udf)
        {
			UdfProperties reportUdf;
            //If the udf is empty use the report's fullname to retrieve the udf from the database
            if (string.IsNullOrEmpty(udf))
                reportUdf = GetUdfFromPath(sessionInfo, reportName.Split(tokens), true);
            else
            {
                reportUdf = JsonConvert.DeserializeObject<UdfProperties>(udf);  // Parse the udf to get the report's id. 

                //Extra check to validate the udf matches the report being requested (ex. in case of converting expressview to advanced report)
                if (reportUdf.ReportType != GetReportTypeFromPath(reportName))
                {
                        reportUdf = GetUdfFromPath(sessionInfo, reportName.Split(tokens), true);
                }
            }

            // query the database for the xml report definition
            try
			{
				DbSqlHandler sql = new DbSqlHandler(db, connectionString);
				sql.Stmt = string.Format("SELECT {0}.content FROM {0} WHERE {0}.id=@reportId", DbTables.Reports);
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@reportId", reportUdf.Id, DbType.String));
				return sql.ExecuteScalarCmd().ToString();
			}
			catch
			{
				return null;
			}
        }

        /** 
		 * Method called by Exago to save the Xml definition of a report
		 * @param sessionInfo is the SessionInfo object as provided by Exago.  
         * @param reportName is the fully qualified path of the report. 
         * @param reportXml is the xml definition of the report. 
         * @return null if save is successful and a string otherwise
         */
        public static string SaveReport(SessionInfo sessionInfo, string reportName, string reportXml)
        {
            UdfProperties reportUdf = GetUdfFromIdThenPath(sessionInfo, reportName, sessionInfo.ReportObject.Id); 
            int reportType = GetReportTypeFromPath(reportName);

            //if the report'sudf is null then it is a new report and saving should insert a new db table row
			if (reportUdf == null)
            {
				string[] pathParts = reportName.Split(tokens);
				UdfProperties parentUdf = GetUdfFromPath(sessionInfo, pathParts.Take(pathParts.Length - 1).ToArray(), false);
                return SaveNewReport(sessionInfo, reportName, reportXml, reportType, parentUdf);
            }
            else
            {
                //otherwise update the existing report definition
				UdfProperties parentUdf = GetUdf(sessionInfo, null, EntityType.ReportFolder, reportUdf.ParentId, null, false);
				return SaveExistingReport(sessionInfo, reportName, reportXml, reportType, reportUdf, parentUdf);
            }
        }

		/** 
		 * Helper method to save a report for the first time
         * @param sessionInfo is the SessionInfo object as provided by Exago.  
         * @param reportName is the name of the report. (no folder path attached) 
         * @param reportXml is the xml definition of the report. 
         * @param reportType is an int representing the type of report.  
         * @param parentUdf is the deserialized UDF string of the folder the report resides in
         * @return null if successful, otherwise returns a string representing an error message
         */
		public static string SaveNewReport(SessionInfo sessionInfo, string reportName, string reportXml, int reportType, UdfProperties parentReportUdf)
        {
			try
			{ 
				DbSqlHandler sql = new DbSqlHandler(db, connectionString);

				// Add the report definition as a row in the Report db
				sql.SqlTxnCmds.Add(new SqlTxnObject(string.Format(
					"INSERT INTO {0} (id, name, content, leaf_flag, type, isDeleted, reportType, ownerId, dateCreated, createdByUserId, dateLastModified, lastModifiedByUserId) " +
					"VALUES (@reportId, @name, @content, @leaf_flag, @type, @deleted, @reportType, @userId, @currentDateTime, @userId, @currentDateTime, @userId); ", DbTables.Reports),
				1));

				// give the creator permission to access and edit the report
				sql.SqlTxnCmds.Add(new SqlTxnObject(string.Format(
					"INSERT INTO {0} (reportId, partyType, partyId, parentId, readOnly, canRename,  IsActive) " +
					"VALUES (@reportId, @partyType, @userId, @parentId, @readOnly, @canRename,  @active); ", DbTables.ReportAccess),
				2));

				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@reportId", sessionInfo.ReportObject.Id, DbType.String));
				string[] pathParts = reportName.Split(tokens);
				string name = pathParts[pathParts.Length - 1];
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@name", name, DbType.String));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@content", reportXml, DbType.String));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@reportType", reportType, DbType.Int32));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@userId", sessionInfo.UserId, DbType.String));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@partyType", PartyType.User, DbType.Int32));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@parentId", parentReportUdf.Id, DbType.String));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@leaf_flag", true, DbType.Boolean));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@readOnly", false, DbType.Boolean));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@canRename", true, DbType.Boolean));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@currentDateTime", DateTime.Now, DbType.DateTime2));
				sql = setDefaultAvailabilityParams(sql, EntityType.ReportFolder);
				sql.ExecTransaction();
            }
			catch (Exception e)
			{
                return e.Message;
            }
            return null;
        }

        /** 
		 * Helper method to save a report that already exists in the database
         * @param sessionInfo is the SessionInfo object as provided by Exago.  
         * @param reportName is the name of the report. (no folder path attached) 
         * @param reportXml is the xml definition of the report. 
         * @param reportType is an int representing the type of report. 
         * @param reportUdf is the deserialized UDF string of the report 
         * @param parentUdf is the deserialized UDF string of the folder the report resides in
         * @return null if successful, otherwise returns a string representing an error message
         */
        public static string SaveExistingReport(SessionInfo sessionInfo, string reportName, string reportXml, int reportType, UdfProperties reportUdf, UdfProperties parentReportUdf)
        {
            try
			{
				DbSqlHandler sql = new DbSqlHandler(db, connectionString);

				// Since the report exists, update its the xml definition in content
				sql.Stmt = string.Format("UPDATE {0} SET content=@content, dateLastModified=@currentDateTime, " +
					"lastModifiedByUserId=@userId WHERE id=@reportId; ", DbTables.Reports);
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@content", reportXml, DbType.String));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@reportId", reportUdf.Id, DbType.String));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@userId", sessionInfo.UserId, DbType.String));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@currentDateTime", DateTime.Now, DbType.DateTime2));
                sql.ExecuteNQCmd();
            }
			catch (Exception e)
			{
                return e.Message;
            }
            return null;
        }

        /** 
		 * Method called by Exago to rename or move a report
         * @param sessionInfo is the SessionInfo object as provided by Exago.  
         * @param reportName is the fully qualified path of the report to rename. 
         * @param newReportName is the fully qualified of the report after renaming.
         * @param udf is the json string set by GetReportListXml of the report.
         * @return null if successful otherwise a string with a user message.
         */
        public static string RenameReport(SessionInfo sessionInfo, string reportName, string newReportName, string udf)
        {
            UdfProperties reportUdf;
            if (string.IsNullOrEmpty(udf)) //if the udf is null for any reason get it from the reportName
                reportUdf = GetUdfFromIdThenPath(sessionInfo, reportName, sessionInfo.ReportObject.Id);
            else
                reportUdf = JsonConvert.DeserializeObject<UdfProperties>(udf);  // Parse the udf to get the report's id. 

			if (!reportUdf.CanRename) //prevent the user from renaming a report they do not have permission to rename
				return sessionInfo.PageInfo.LanguageDict.Dict["CannotRenameReport"].Html;

			string[] newPathParts = newReportName.Split(tokens);
			UdfProperties newParentFolderUdf = GetUdfFromPath(sessionInfo, newPathParts.Take(newPathParts.Length - 1).ToArray(), false);
            string newName = newPathParts[newPathParts.Length - 1];

            //Check if the folder location changed or just the report name
            if (reportUdf.ParentId == newParentFolderUdf.Id)
                return UpdateReportName(sessionInfo, newName, reportUdf.Id);	// if the location is unchanged update the name
			else
                return UpdateReportParentAndName(sessionInfo, newName, reportUdf, newParentFolderUdf);	// If the location is changed update both the name and the parent
		}

        /** 
		 * Helper method to change the name of a report that already exists in the database
         * @param sessionInfo is the SessionInfo object as provided by Exago.  
         * @param name is the new name of the report. (no folder path attached) 
         * @param reportId the report's identifier in the database.
         * @return null if successful, otherwise returns a string representing an error message
         */
        public static string UpdateReportName(SessionInfo sessionInfo, string name, string reportId)
        {
            try
			{
				DbSqlHandler sql = new DbSqlHandler(db, connectionString);

				// Since the report exists, update its name column
				sql.Stmt = string.Format("UPDATE {0} SET name=@name, dateLastModified=@currentDateTime, lastModifiedByUserId=@userId " +
					"WHERE id=@reportId ", DbTables.Reports);
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@name", name, DbType.String));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@reportId", reportId, DbType.String));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@userId", sessionInfo.UserId, DbType.String));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@currentDateTime", DateTime.Now, DbType.DateTime2));
                sql.ExecuteNQCmd();
            }
			catch (Exception e)
			{
                return e.Message;
            }
            return null;
        }

        /** 
		 * Helper method to change the name and location of a report that already exists in the database
         * @param sessionInfo is the SessionInfo object as provided by Exago.  
         * @param name is the new name of the report. (no folder path attached) 
         * @param reportUdf is the deserialized UDF string of the report 
         * @param parentUdf is the deserialized UDF string of the folder the report will be moved  to
         * @return null if successful otherwise a string with a user message.
         */
        public static string UpdateReportParentAndName(SessionInfo sessionInfo, string name, UdfProperties reportUdf, UdfProperties newParentFolderUdf)
        {
			try
			{
				DbSqlHandler sql = new DbSqlHandler(db, connectionString);

				// First update the report name
				sql.SqlTxnCmds.Add(new SqlTxnObject(string.Format(
					"UPDATE {0} SET name=@name, dateLastModified=@currentDateTime, lastModifiedByUserId=@userId " +
					"WHERE id=@reportId; ", DbTables.Reports),
				1));

				// Update the report's parent for that user. 
				// If they didn't have user level access give them user level access with an updated parentId, otherwise update the existing record
				if (reportUdf.partyType != (int)PartyType.User)
				{
					sql.SqlTxnCmds.Add(new SqlTxnObject(string.Format(
                        "INSERT INTO {0} (reportId, partyType, partyId, parentId, readOnly, canRename, IsActive) " +
                        "VALUES (@reportId, @partyType, @userId, @parentId, @readOnly, @canRename, @active); ", DbTables.ReportAccess),
					2));
					sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@readOnly", false, DbType.Boolean));
					sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@canRename", true, DbType.Boolean));
				}
				else
				{
					sql.SqlTxnCmds.Add(new SqlTxnObject(string.Format(
						"UPDATE {0} SET parentId=@parentId " +
						"WHERE reportId=@reportId AND partyType=@partyType AND partyId=@userId AND isActive=@active; ", DbTables.ReportAccess),
					2));
				}
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@currentDateTime", DateTime.Now, DbType.DateTime2));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@name", name, DbType.String));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@reportId", reportUdf.Id, DbType.String));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@userId", sessionInfo.UserId, DbType.String));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@partyType", PartyType.User, DbType.Int32));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@parentId", newParentFolderUdf.Id, DbType.String));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@active", true, DbType.Boolean));
				sql.ExecTransaction();
            }
			catch (Exception e)
			{
                return e.Message;
            }
            return null;
        }

        /** 
		 * Method called by Exago to delete a report
         * @param sessionInfo is the SessionInfo object as provided by Exago.  
         * @param reportName is the fully qualified path of the report to delete. 
         * @return null if successful, otherwise returns a string representing an error message
         */
        public static string DeleteReport(SessionInfo sessionInfo, string reportName, string udf)
        {

            UdfProperties reportUdf;
            if (udf == null)
                reportUdf = GetUdfFromIdThenPath(sessionInfo, reportName, sessionInfo.ReportObject.Id);
            else
                reportUdf = JsonConvert.DeserializeObject<UdfProperties>(udf);

			try
			{
				DbSqlHandler sql = new DbSqlHandler(db, connectionString);

                // If the user is the report's owner, update the IsDeleted report to delete it for everyone
                if (reportUdf.OwnerId == sessionInfo.UserId)
                {
                    sql.Stmt = string.Format("UPDATE {0} SET isDeleted=@deleted, dateLastModified=@currentDateTime, " +
                        "lastModifiedByUserId=@userId WHERE id=@reportId; ", DbTables.Reports);
                    sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@deleted", true, DbType.Boolean));
                    sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@currentDateTime", DateTime.Now, DbType.DateTime2));
                }
                else	// Otherwise, deactivate the permissions for just the current user
				{
					sql.Stmt = string.Format("UPDATE {0} SET isActive=@active WHERE reportId=@reportId " +
						"AND partyType=@partyType AND partyId=@userId; ", DbTables.ReportAccess);
                    sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@active", false, DbType.Boolean));
                }
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@reportId", reportUdf.Id, DbType.String));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@partyType", PartyType.User, DbType.Int32));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@userId", sessionInfo.UserId, DbType.String));
                sql.ExecuteNQCmd();
            }
            catch (Exception e)
            {
                return e.Message;
            }
            return null;
        }

        /** 
		 * Method called by Exago to create a new folder
         * @param sessionInfo is the SessionInfo object as provided by Exago.  
         * @param reportName is the fully qualified path of the folder. 
         * @param parentUdf is the json string set by GetReportListXml of the parent folder the folder resides in.
         * @return null if successful otherwise a string with a user message.
         */
        public static string AddFolder(SessionInfo sessionInfo, string folderName)
        {
            string[] pathParts = folderName.Split(tokens);
            string[] parentPathParts = (pathParts.Length - 1 == 0 ? null : pathParts.Take(pathParts.Length - 1).ToArray());

            //check the parentUDF to determine if this is a root folder
            object parentFolderId = DBNull.Value;
            if (parentPathParts != null)
            {
                UdfProperties parentFolderUdf = GetUdfFromPath(sessionInfo, parentPathParts, false);
                parentFolderId = parentFolderUdf.Id;
            }

			try
			{
				DbSqlHandler sql = new DbSqlHandler(db, connectionString);

				// Insert a row to the Reports table of the db; set leaf_flag to false to indicate a folder
				sql.SqlTxnCmds.Add(new SqlTxnObject(string.Format(
                    "INSERT INTO {0} (id, name, leaf_flag, type, isDeleted, ownerId, dateCreated, createdByUserId, dateLastModified, lastModifiedByUserId) " +
                    "VALUES (@folderId, @name, @leaf_flag, @type, @deleted, @userId, @currentDateTime, @userId, @currentDateTime, @userId); ", DbTables.Reports),
                1));

				sql.SqlTxnCmds.Add(new SqlTxnObject(string.Format(
                    "INSERT INTO {0} (reportId, partyType, partyId, parentId, readOnly, canRename,  IsActive) " +
                    "VALUES (@folderId, @partyType, @userId, @parentId, @readOnly, @canRename,  @active); ", DbTables.ReportAccess),
				2));

				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@folderId", Guid.NewGuid().ToString(), DbType.String));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@name", pathParts[pathParts.Length - 1], DbType.String));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@leaf_flag", false, DbType.Boolean));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@userId", sessionInfo.UserId, DbType.String));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@partyType", PartyType.User, DbType.Int32));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@parentId", parentFolderId, DbType.String));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@readOnly", false, DbType.Boolean));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@canRename", true, DbType.Boolean));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@currentDateTime", DateTime.Now, DbType.DateTime2));
				sql = setDefaultAvailabilityParams(sql, EntityType.ReportFolder);
				sql.ExecTransaction();
            }
            catch (Exception e)
            {
                return e.Message;
            }
            return null;
        }

        /** 
		 * Method called by Exago to rename a folder
         * @param sessionInfo is the SessionInfo object as provided by Exago.  
         * @param name is the fully qualified path of the folder to rename. 
         * @param newtName is the fully qualified of the folder after renaming.
         * @param udf is the json string set by GetReportListXml of the folder.
         * @return null if successful otherwise a string with a user message.
         */
        public static string RenameFolder(SessionInfo sessionInfo, string oldName, string newName)
        {
            string[] pathParts = oldName.Split(tokens);
            string[] newPathParts = newName.Split(tokens);
            UdfProperties folderUdf = GetUdfFromPath(sessionInfo, pathParts, false);
            //Check that the user has permission to update the folder's name
            if (!folderUdf.CanRename)
                return "You do not have permissions to rename the report.";

            try
            {
                DbSqlHandler sql = new DbSqlHandler(db, connectionString);

				// Update the folder's name in the database
				sql.Stmt = string.Format("UPDATE {0} SET name=@name, dateLastModified=@currentDateTime, " +
					"lastModifiedByUserId=@userId WHERE id=@folderId ", DbTables.Reports);
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@name", newPathParts[pathParts.Length - 1], DbType.String));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@folderId", folderUdf.Id, DbType.String));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@userId", sessionInfo.UserId, DbType.String));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@currentDateTime", DateTime.Now, DbType.DateTime2));
                sql.ExecuteNQCmd();
            }
            catch (Exception e)
            {
                return e.Message;
            }
            return null;
        }

        /** 
		 * Method called by Exago to move a folder
         * @param sessionInfo is the SessionInfo object as provided by Exago.  
         * @param oldName is the fully qualified path of the folder to be moved. 
         * @param newName is the fully qualified path for the folder to be changed to.
         * @return null if successful otherwise a string with a user message.
         */
        public static string MoveFolder(SessionInfo sessionInfo, string oldName, string newName)
        {
            UdfProperties folderUdf = GetUdfFromPath(sessionInfo, oldName.Split(tokens), false);
            if (!folderUdf.CanRename)
            {
                return sessionInfo.PageInfo.LanguageDict.Dict["CannotMoveFolder"].Html;
            }
            string[] parentFolderPath = newName.Split(tokens); 
            UdfProperties newParentFolderUdf = (parentFolderPath.Length == 1 ? null : GetUdfFromPath(sessionInfo, parentFolderPath.Take(parentFolderPath.Length - 1).ToArray(), false));

            //If they didn't have user level access give them user level access with an updated parentId, otherwise update the existing access record
            object parentFolderId = DBNull.Value;
            if (newParentFolderUdf != null)
                parentFolderId = newParentFolderUdf.Id;

			try
			{
				DbSqlHandler sql = new DbSqlHandler(db, connectionString);

				if (folderUdf.partyType != (int)PartyType.User)
                {
                    sql.Stmt = string.Format(
                        "INSERT INTO {0} (reportId, partyType, partyId, parentId, readOnly, canRename,  IsActive) " +
                        "VALUES (@folderId, @partyType, @userId, @parentId, @readOnly, @canRename,  @active); ", DbTables.ReportAccess);
                    sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@readOnly", true, DbType.Boolean));
                    sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@canRename", folderUdf.CanRename, DbType.Boolean));
                    
                }
                else
                {
                    sql.Stmt = string.Format(
                        "UPDATE {0} SET parentId=@parentId " +
                        "WHERE reportId=@folderId AND partyType=@partyType AND partyId=@userId AND IsActive=@active; ", DbTables.ReportAccess);
                }
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@partyType", PartyType.User, DbType.Int32));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@userId", sessionInfo.UserId, DbType.String));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@parentId", parentFolderId, DbType.String));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@folderId", folderUdf.Id, DbType.String));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@active", true, DbType.Boolean));
                sql.ExecuteNQCmd();
            }
            catch (Exception e)
            {
                return e.Message;
            }
            return null;
        }

        /** 
		 * Method called by Exago to delete a folder
         * @param sessionInfo is the SessionInfo object as provided by Exago.  
         * @param reportName is the fully qualified path of the folder to delete. 
         * @return null if successful, otherwise returns a string representing an error message
         */
        public static string DeleteFolder(SessionInfo sessionInfo, string folderName)
        {
			string[] pathParts = folderName.Split(tokens);
			UdfProperties folderUdf = GetUdfFromPath(sessionInfo, pathParts, false);

			try
			{
                DbSqlHandler sql = new DbSqlHandler(db, connectionString);

                //If the user is the folder's owner update the IsDeleted folder to delete it for everyone
                if (folderUdf.OwnerId == sessionInfo.UserId)
                {
                    sql.Stmt = string.Format("UPDATE {0} SET isDeleted=@deleted, dateLastModified=@currentDateTime, lastModifiedByUserId=@userId " +
                        "WHERE id=@reportId; ", DbTables.Reports);
                    sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@deleted", true, DbType.Boolean));
                    sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@currentDateTime", DateTime.Now, DbType.DateTime2));
                }
                else	// Otherwise, deactivate the folder for just that one user
				{
					sql.Stmt = string.Format("UPDATE {0} SET isActive=@active " +
                        "WHERE reportId=@reportId AND partyType=@partyType AND partyId=@userId;", DbTables.ReportAccess);
                    sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@active", false, DbType.Boolean));
                }
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@reportId", folderUdf.Id, DbType.String));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@partyType", PartyType.User, DbType.Int32));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@userId", sessionInfo.UserId, DbType.String));
                sql.ExecuteNQCmd();
            }
            catch (Exception e)
            {
                return e.Message;
            }
            return null;
        }

        /** 
		 * Method called by Exago to check if a folder exists.
         * @param sessionInfo is the SessionInfo object as provided by Exago.  
         * @param reportName is the fully qualified path of the folder.
         * @return a boolean indicating the folder's existence.
         */
        public static bool ExistFolder(SessionInfo sessionInfo, string folderName)
        {
            UdfProperties folderUdf = GetUdfFromPath(sessionInfo, folderName.Split(tokens), false);
            return (folderUdf != null);
        }

        /** 
		 * Method called by Exago to populate the list of document templates available to the active user
         * @param sessionInfo is the SessionInfo object as provided by Exago.  
         * @return list of template names as strings
         */
        public static List<string> GetTemplateList(SessionInfo sessionInfo)
        {
			List<string> tmplList = new List<string>();

			try
			{
				DbSqlHandler sql = new DbSqlHandler(db, connectionString);

				sql.Stmt = string.Format(
					"SELECT DISTINCT {0}.name " +
					"FROM {0} " +
					"INNER JOIN {1} ON {0}.id={1}.reportId " +
					"WHERE " +
						availabilitySqlClause +
						permissionSqlClause +
					"ORDER BY name ASC", DbTables.Reports, DbTables.ReportAccess);
				sql = setDefaultPermissionParams(sessionInfo, sql);
				sql = setDefaultAvailabilityParams(sql, EntityType.DocumentTemplate);

				DataSet ds = sql.ExecuteReadCmd();

				foreach (DataRow dr in ds.Tables[0].Rows)
					tmplList.Add(dr.Field<string>("name"));
			}
			catch
			{
				return null;
			}
			return tmplList;
		}

        /** 
		 * Method called by Exago to get the content of a document template
         * @param sessionInfo is the SessionInfo object as provided by Exago.  
         * @param templateName is the name of the template being retrieved
         * @return a byte array of the document template's content
         */
        public static byte[] GetTemplate(SessionInfo sessionInfo, string templateName)
        {
			byte[] tmpl = new byte[0];
			try
			{
				DbSqlHandler sql = new DbSqlHandler(db, connectionString);

				//Query the db for a template's bit content
				sql.Stmt = string.Format(
					"SELECT {0}.bit_content " +
					"FROM {0} " +
					"INNER JOIN {1} ON {0}.id={1}.reportId " +
					"WHERE " +
						"{0}.name=@templateName AND " +
						availabilitySqlClause +
						permissionSqlClause, DbTables.Reports, DbTables.ReportAccess);
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@templateName", templateName, DbType.String));
				sql = setDefaultPermissionParams(sessionInfo, sql);
				sql = setDefaultAvailabilityParams(sql, EntityType.DocumentTemplate);
				tmpl = (byte[])sql.ExecuteScalarCmd();
			}
			catch
			{
				return null;
			}
			return tmpl;
		}

		/** 
		 * Method called by Exago to store the content of a document template
         * @param sessionInfo is the SessionInfo object as provided by Exago.  
         * @param templateName is the name of the template being saved
         * @param templateData is a byte array of the content of the template
         * @return null upon success and error message otherwise
         */
		public static string SaveTemplate(SessionInfo sessionInfo, string templateName, byte[] templateData)
        {
			try
			{
				DbSqlHandler sql = new DbSqlHandler(db, connectionString);

				//Add a row to the db to define the document template and give the user access to the Document Template
				sql.SqlTxnCmds.Add(new SqlTxnObject(string.Format(
					"INSERT INTO {0} (id, name, bit_content, type, isDeleted, ownerId, dateCreated, createdByUserId, dateLastModified, lastModifiedByUserId) " +
					"VALUES (@templateId, @name, @bit_content, @type, @deleted, @userId, @currentDateTime, @userId, @currentDateTime, @userId); ", DbTables.Reports),
				1));
                sql.SqlTxnCmds.Add(new SqlTxnObject(string.Format(
					"INSERT INTO {0} (reportId, partyType, partyId, isActive) " +
					"VALUES (@templateId, @partyType, @userId, @active); ", DbTables.ReportAccess),
                2));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@templateId", Guid.NewGuid().ToString(), DbType.String));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@name", templateName, DbType.String));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@bit_content", templateData, DbType.Binary));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@userId", sessionInfo.UserId, DbType.String));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@partyType", PartyType.User, DbType.Int32));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@currentDateTime", DateTime.Now, DbType.DateTime2));
				sql = setDefaultAvailabilityParams(sql, EntityType.DocumentTemplate);
				sql.ExecTransaction();
            }
            catch (Exception e)
            {
                return e.Message;
            }
            return null;
        }

        /** 
		 * Method called by Exago to populate the list of themes available to the active user
         * @param sessionInfo is the SessionInfo object as provided by Exago.
         * @param themeType is a string representing the type of theme (e.g. Charts, Maps, etc.)  
         * @return list of themes names as strings
         */
        public static List<string> GetThemeList(SessionInfo sessionInfo, string themeType)
        {
			List<string> themeList = new List<string>();

			try
			{
				DbSqlHandler sql = new DbSqlHandler(db, connectionString);

				//Query the db for a all of the templates available to the user
				sql.Stmt = string.Format(
					"SELECT DISTINCT {0}.name " +
					"FROM {0} " +
					"INNER JOIN {1} ON {0}.Id={1}.reportId " +
					"WHERE " +
						availabilitySqlClause +
						"{0}.themeType=@themeType AND " +
						permissionSqlClause +
					"ORDER BY name ASC", DbTables.Reports, DbTables.ReportAccess);
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@themeType", themeType, DbType.String));
				sql = setDefaultPermissionParams(sessionInfo, sql);
				sql = setDefaultAvailabilityParams(sql, EntityType.Theme);

				DataSet ds = sql.ExecuteReadCmd();

				foreach (DataRow dr in ds.Tables[0].Rows)
					themeList.Add(dr.Field<string>("name"));
			}
			catch
			{
				return null;
			}
			return themeList;
		}

		/** 
		 * Method called by Exago to check if a theme exists
         * @param sessionInfo is the SessionInfo object as provided by Exago.
         * @param themeType is a string representing the type of theme (e.g. Charts, Maps, etc.)  
         * @param themeName is the name of the theme being queried
         * @return a boolean indicating the theme's existence; false upon failure
         */
		public static bool ExistTheme(SessionInfo sessionInfo, string themeType, string themeName)
        {
			bool themeExists = false;

			try
			{
				DbSqlHandler sql = new DbSqlHandler(db, connectionString);

				sql.Stmt = string.Format(
					"SELECT {0}.name " +
					"FROM {0} " +
					"INNER JOIN {1} ON {0}.id={1}.reportId " +
					"WHERE " +
						availabilitySqlClause +
						"{0}.themeType=@themeType AND " +
						"{0}.name=@themeName AND " +
						permissionSqlClause, DbTables.Reports, DbTables.ReportAccess);
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@themeType", themeType, DbType.String));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@themeName", themeName, DbType.String));
				sql = setDefaultPermissionParams(sessionInfo, sql);
				sql = setDefaultAvailabilityParams(sql, EntityType.Theme);
				themeExists = !string.IsNullOrEmpty((string)sql.ExecuteScalarCmd());
			}
			catch
			{
				return false;
			}
			return themeExists;
		}

		/** 
		 * Method called by Exago to retrieve the content of a theme
         * @param sessionInfo is the SessionInfo object as provided by Exago.
         * @param themeType is a string representing the type of theme (e.g. Charts, Maps, etc.)  
         * @param themeName is the name of the theme being retrieved
         * @return a string representing the theme's content
         */
		public static string GetThemeXml(SessionInfo sessionInfo, string themeType, string themeName)
        {
			string themeXml = null;

			try
			{
				DbSqlHandler sql = new DbSqlHandler(db, connectionString);

				sql.Stmt = string.Format(
					"SELECT {0}.content " +
					"FROM {0} " +
					"INNER JOIN {1} ON {0}.id={1}.reportId " +
					"WHERE " +
						availabilitySqlClause +
						"{0}.themeType=@themeType AND " +
						"{0}.name=@themeName AND " +
						permissionSqlClause, DbTables.Reports, DbTables.ReportAccess);
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@themeType", themeType, DbType.String));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@themeName", themeName, DbType.String));
				sql = setDefaultPermissionParams(sessionInfo, sql);
				sql = setDefaultAvailabilityParams(sql, EntityType.Theme);
				themeXml = sql.ExecuteScalarCmd().ToString();
			}
			catch (Exception e)
			{
				string s = e.Message;
			}

			return themeXml;
		}

		/** 
		 * Method called by Exago to store the content of a theme
         * @param sessionInfo is the SessionInfo object as provided by Exago.
         * @param themeType is a string representing the type of theme (e.g. Charts, Maps, etc.)  
         * @param themeName is the name of the theme being saved
         * @param themeXml is a byte array of the content of the template
         * @return null if successful, otherwise returns a string representing an error message
         */
		public static string SaveTheme(SessionInfo sessionInfo, string themeType, string themeName, string themeXml)
        {
			try
			{
				DbSqlHandler sql = new DbSqlHandler(db, connectionString);

				//Add a row to the db to define the theme and give the user access 
                sql.SqlTxnCmds.Add(new SqlTxnObject(string.Format(
					"INSERT INTO {0} (id, name, content, type, isDeleted, themeType, ownerId, dateCreated, createdByUserId, dateLastModified, lastModifiedByUserId) " +
					"VALUES (@themeId, @name, @content, @type, @deleted, @themeType, @userId, @currentDateTime, @userId, @currentDateTime, @userId); ", DbTables.Reports),
				1));
                sql.SqlTxnCmds.Add(new SqlTxnObject(string.Format(
					"INSERT INTO {0} (reportId, partyType, partyId, ReadOnly, isActive) " +
					"VALUES (@themeId, @partyType, @userId, @ro, @active); ", DbTables.ReportAccess),
				2));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@themeId", Guid.NewGuid().ToString(), DbType.String));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@name", themeName, DbType.String));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@ro", true, DbType.Boolean));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@content", themeXml, DbType.String));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@themeType", themeType, DbType.String));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@userId", sessionInfo.UserId, DbType.String));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@partyType", PartyType.User, DbType.Int32));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@currentDateTime", DateTime.Now, DbType.DateTime2));
				sql = setDefaultAvailabilityParams(sql, EntityType.Theme);
				sql.ExecTransaction();
			}
			catch (Exception e)
			{
				return e.Message;
			}
			return null;
		}

		/**
		 * By extracting the report name extension, determine the report type 
		 * and return ReportTypes enumeration object value
		 * @param reportName is the name of the report, including "file extension"
		 * @return ReportTypes enumeration value
		 */
        public static int GetReportTypeFromPath(string reportName)
        {
            string suffix = reportName.Substring(reportName.Length - 4);
            switch (suffix)
            {
                case (".wrq"):
                    return (int)ReportType.ExpressView;
                case (".wrd"):
                    return (int)ReportType.Dashboard;
                case (".wrc"):
                    return (int)ReportType.Chained;
                case (".wrv"):
                    return (int)ReportType.Visualization;
                case (".wrx"):
                    return (int)ReportType.Express;
                default:
                    return (int)ReportType.Standard;
            }
        }

		/**
		 * Get the UDFProperties associated with a specific EntityType item
         * @param sessionInfo is the SessionInfo object as provided by Exago.
		 * @param entityName is the name of the report, folder, etc
		 * @param entityType is the EntityType for the entity
		 * @param entityId is the unique identifier for the entity
		 * @param parentId is the unique identifier for the entity's parent entity
		 * @param isReport is a boolean indicating whether this is a report or not
		 * @return UdfProperties object containing data unique to this EntityType item
		 */
		public static UdfProperties GetUdf(SessionInfo sessionInfo, string entityName, EntityType entityType, string entityId, string parentId, bool isReport = false)
		{
			try
			{
				DbSqlHandler sql = new DbSqlHandler(db, connectionString);

				sql.Stmt = string.Format(
					"SELECT id, name, leaf_flag, parentId, ownerId, reportType, " +
					"    MIN( CASE WHEN readOnly=@ro THEN 1 ELSE 0 END ) AS readOnly, " +
					"    MAX( sortOrder ) AS sortOrder, " +
					"    MAX( CASE WHEN canRename=@canRename THEN 1 ELSE 0 END ) AS canRename, " +
                    "    MAX( partyType ) AS partyType " +
					"FROM ( " +
						"SELECT {0}.id, {0}.name, {0}.leaf_flag, {0}.ownerId, {0}.reportType, " +
							"{1}.readOnly, {1}.parentId, {1}.sortOrder, {1}.canRename, {1}.partyType  " +
						"FROM {0} " +
						"INNER JOIN {1} ON {0}.id={1}.reportId " +
						"INNER JOIN " +
							"( " +
								"SELECT {1}.reportId, MAX(partyType) AS partyType " +
								"FROM {1} " +
								"WHERE " +
								"{1}.isActive=@active AND " +
								permissionSqlClause +
								"GROUP BY {1}.reportId " +
							") AS b " +
							"ON {1}.reportId=b.reportId AND {1}.partyType=b.partyType " +
						"WHERE " +
							(string.IsNullOrEmpty(entityName) ? "" :
								(string.IsNullOrEmpty(entityId) ? 
									"{0}.name=@entityName AND " : "" ) ) +
							(string.IsNullOrEmpty(entityId) ? "" : "{0}.id=@entityId AND ") +
							(entityType.Equals(EntityType.ReportFolder) ?
								(string.IsNullOrEmpty(entityId) ? 
									"{1}.parentId " + (string.IsNullOrEmpty(parentId) ? " IS NULL " : "=@parentId ") + " AND " : "" ) +
								"{0}.leaf_flag=@leaf_flag AND " : "" ) +
							availabilitySqlClause +
							permissionSqlClause +
					") AS ReportListXML " +
					"GROUP BY id, name, leaf_flag, parentId, ownerId, reportType " +
					"ORDER BY sortOrder DESC, leaf_flag ASC, name ASC", DbTables.Reports, DbTables.ReportAccess);

				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@parentId", parentId, DbType.String));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@ro", true, DbType.Boolean));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@canRename", true, DbType.Boolean));
                sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@entityName", entityName, DbType.String));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@entityId", entityId, DbType.String));
				sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@leaf_flag", isReport, DbType.Boolean));
				sql = setDefaultPermissionParams(sessionInfo, sql);
				sql = setDefaultAvailabilityParams(sql, entityType);

				DataSet ds = sql.ExecuteReadCmd();

				if (ds == null || ds.Tables[0].Rows.Count == 0)
					return null;

				DataRow dr = ds.Tables[0].Rows[0];
				return new UdfProperties(dr.Field<string>("id"), dr.Field<string>("parentId"), dr.Field<int?>("sortOrder"), dr.Field<string>("ownerId"),
					Convert.ToBoolean(dr["canRename"]), dr.Field<int?>("reportType"), dr.Field<int?>("partyType"));
			}
			catch 
			{
				return null;
			}
		}

		/**
		 * Navigate path, if any, and get and return udf value based on report name
         * @param sessionInfo is the SessionInfo object as provided by Exago.
		 * @param entityPath is a string array of parsed path elements
		 * @param isReport is a boolean true if report, false if folder
		 * @return UdfProperties object containing data unique to this report/folder
		 */
		public static UdfProperties GetUdfFromPath(SessionInfo sessionInfo, string[] entityPath, bool isReport)
        {
            //if the path length is 1 then the path is for a root folder. Get the Udf by name knowing it is a folder with null parentId
			//this scenario should fall straight through and not need to be specifically implemented here

            //Start with the root folder in the path and then get the Id until the next to last part of the path. 
            string id = null;
			try
			{
				DbSqlHandler sql = new DbSqlHandler(db, connectionString);

				for (int i = 0; i < entityPath.Length - 1; i++)
				{
					sql.Clear();
					sql.Stmt = string.Format(
						"SELECT DISTINCT {0}.id " +
						"FROM {0} " +
						"INNER JOIN {1} ON {0}.id={1}.reportId " +
						"WHERE " +
							"{0}.name=@entityName AND " +
							"{1}.parentId " + (i == 0 ? "IS NULL " : "=@parentId ") + " AND " +
							"{0}.leaf_flag=@leaf_flag AND " +
							availabilitySqlClause +
							permissionSqlClause, DbTables.Reports, DbTables.ReportAccess);
					if (i > 0)
						sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@parentId", id, DbType.String));

					sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@entityName", entityPath[i], DbType.String));
					sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@leaf_flag", (isReport ? (i == entityPath.Length - 1 ? true: false) : false) , DbType.Boolean));
					sql = setDefaultPermissionParams(sessionInfo, sql);
					sql = setDefaultAvailabilityParams(sql, EntityType.ReportFolder);
					id = sql.ExecuteScalarCmd().ToString();
				}
			}
			catch { /* not sure what to return in case of failure */ }

			//Use the Id of the next to last item as the parentId
			return GetUdf(sessionInfo, entityPath[entityPath.Length - 1], EntityType.ReportFolder, null, id, isReport);
		}

		/**
		 * Attempts to determine UDF data from the provided entity id and, if not found, subsequently determine UDF data
		 * based the report path and path name
         * @param sessionInfo is the SessionInfo object as provided by Exago.
		 * @param reportName is the name of the report, including folder path
		 * @param id is the unique identifier for the report
		 */
		public static UdfProperties GetUdfFromIdThenPath(SessionInfo sessionInfo, string reportName,  string id)
        {
			UdfProperties reportUdf = GetUdf(sessionInfo, reportName, EntityType.ReportFolder, id, null, true);

			if (reportUdf == null)	//Additional check in case the udf is not found by id
                reportUdf = GetUdfFromPath(sessionInfo, reportName.Split(tokens), true);

            return reportUdf;
        }

		/**
		 * Helper object that provides a SQL clause that determines if an entity is available (not deleted, not inactive, and of a specific type)
		 */ 
		private static string availabilitySqlClause = string.Format(
			"{0}.isDeleted=@deleted AND " +
			"{0}.type=@type AND " +
			"{1}.isActive=@active AND ", 
			DbTables.Reports, DbTables.ReportAccess);

		/**
		 * Helper method used to set default SQL parameters for availability
		 * @param sql is the DbSqlHandler object containing the associated SQL statement
		 * @param type is the EntityType of the object whose availability is being assessed
		 * @return a modified DbSqlHandler object
		 */ 
		private static DbSqlHandler setDefaultAvailabilityParams(DbSqlHandler sql, EntityType type)
		{
			sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@deleted", false, DbType.Boolean));
			sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@active", true, DbType.Boolean));
			sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@type", type, DbType.Int32));
			return sql;
		}

		/**
		 * Helper object that provides a SQL clause that determines if a party has the appropriate permissions to access an entity
		 */
		private static string permissionSqlClause = string.Format(
			"( " +
				"( {0}.partyType=@everyone ) OR " +
				"( {0}.partyType=@class AND {0}.partyId=@classId ) OR " +
				"( {0}.partyType=@company AND {0}.partyId=@companyId ) OR " +
				"( {0}.partyType=@user AND {0}.partyId=@userId ) " +
			") ",
			DbTables.ReportAccess);

		/**
		 * Helper method used to set default SQL parameters for permissions
         * @param sessionInfo is the SessionInfo object as provided by Exago.
		 * @param sql is the DbSqlHandler object containing the associated SQL statement
		 * @return a modified DbSqlHandler object
		 */
		private static DbSqlHandler setDefaultPermissionParams(SessionInfo sessionInfo, DbSqlHandler sql)
		{
			sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@everyone", PartyType.Everyone, DbType.Int32));
			sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@class", PartyType.Class, DbType.Int32));
			sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@classId", sessionInfo.GetParameter("classId").Value, DbType.String));
			sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@company", PartyType.Company, DbType.Int32));
			sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@companyId", sessionInfo.GetParameter("companyKeyId").Value, DbType.String));
			sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@user", PartyType.User, DbType.Int32));
			sql.SqlParams.Add(DbObjectFactory.GetParameter(db, "@userId", sessionInfo.UserId, DbType.String));
			return sql;
		}

		/**
		* These differentiate the various types of objects that are saved via Folder Management.  Note
		* that reports and folders are both saved as ReportFolder = 0.
		*/
		public enum EntityType : int { ReportFolder = 0, DocumentTemplate = 1, Theme = 2 };

		/**
		 * Any XML document node, for the purpose of rebuilding an XML document from database records
		 */
		public class Entity
		{
			public string Id;
			public string Name;
			public bool LeafFlag;
			public bool ReadOnlyFlag;
			public string UDF;
			public UdfProperties udfProperties;
			public List<Entity> Children = new List<Entity>();

			// object constructor
			public Entity(string id, string name, bool leafFlag, bool readOnlyFlag, string parentId, int? sortOrder, string ownerId, bool canRename,  int? reportType, int? partyType, List<Entity> children = null)
			{
				this.Id = id;
				this.Name = name;
				this.LeafFlag = leafFlag;
				this.ReadOnlyFlag = readOnlyFlag;
				this.udfProperties = new UdfProperties(id, parentId, sortOrder, ownerId, canRename, reportType, partyType);
				this.UDF = this.udfProperties.ToString();
				this.Children = children;
			}
		}
	}
}